/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Authors: Christian Hitz, digitalSTROM AG <christian.hitz@digitalstrom.com>
             Michael Tross, digitalSTROM AG <michael.tross@digitalstrom.com>
             Reto Flütsch, digitalSTROM AG <reto.fluetsch@digitalstrom.com>

    This file is part of vdSM.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.
    

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vdSM. If not, see <http://www.gnu.org/licenses/>.
*/

#include "scene.h"
#include "button.h"
#include "datamodel.h"

Q_DEFINE_THIS_MODULE(scene)

/**
 * Scene table.
 * X-axis: off scene, on scene, scene 2, scene 3, scene 4, increment, decrement, increment continue, decrement continue, stop
 * Y-axis: button ID
 */
static const Scene l_scenes[BUTTON_LAST][10] = {
    { T0_S0, T0_S1, T0_S2, T0_S3, T0_S4, INC_S, DEC_S, INC_S, DEC_S, STOP_S },
    { T1_S0, T1_S1, T0_S2, T0_S3, T0_S4, T1_INC, T1_DEC, T1234_CONT, T1234_CONT, T1_STOP_S },
    { T2_S0, T2_S1, T0_S2, T0_S3, T0_S4, T2_INC, T2_DEC, T1234_CONT, T1234_CONT, T2_STOP_S },
    { T3_S0, T3_S1, T0_S2, T0_S3, T0_S4, T3_INC, T3_DEC, T1234_CONT, T1234_CONT, T3_STOP_S },
    { T4_S0, T4_S1, T0_S2, T0_S3, T0_S4, T4_INC, T4_DEC, T1234_CONT, T1234_CONT, T4_STOP_S },
    { T0_S0, T0_S1, T0_S2, T0_S3, T0_S4, INC_S, DEC_S, INC_S, DEC_S, STOP_S },
    { T1E_S0, T1E_S1, T1_S2, T1_S3, T1_S4, INC_S, DEC_S, INC_S, DEC_S, STOP_S },
    { T2E_S0, T2E_S1, T2_S2, T2_S3, T2_S4, INC_S, DEC_S, INC_S, DEC_S, STOP_S },
    { T3E_S0, T3E_S1, T3_S2, T3_S3, T3_S4, INC_S, DEC_S, INC_S, DEC_S, STOP_S },
    { T4E_S0, T4E_S1, T4_S2, T4_S3, T4_S4, INC_S, DEC_S, INC_S, DEC_S, STOP_S },
    { T1_S0, T1_S1, T1_S2, T1_S3, T1_S4, T1_INC, T1_DEC, T1234_CONT, T1234_CONT, T1_STOP_S },
    { T2_S0, T2_S1, T2_S2, T2_S3, T2_S4, T2_INC, T2_DEC, T1234_CONT, T1234_CONT, T2_STOP_S },
    { T3_S0, T3_S1, T3_S2, T3_S3, T3_S4, T3_INC, T3_DEC, T1234_CONT, T1234_CONT, T3_STOP_S },
    { T4_S0, T4_S1, T4_S2, T4_S3, T4_S4, T4_INC, T4_DEC, T1234_CONT, T1234_CONT, T4_STOP_S },
    { T0_S0, T0_S1, T0_S2, T0_S3, T0_S4, INC_S, DEC_S, INC_S, DEC_S, STOP_S },
    { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF }, /* server handled device */
};

/*..........................................................................*/
Scene Scene_GetTabEntry(uint8_t buttonId, uint8_t idx)
{
    Q_ASSERT(buttonId < BUTTON_LAST);
    return l_scenes[buttonId][idx];
}

/*..........................................................................*/
bool Scene_IsOffScene(uint8_t sceneId)
{
    uint8_t buttonId;
    if (sceneId == AUTO_OFF) {
        return TRUE;
    }
    for (buttonId = 0; buttonId < BUTTON_LAST; ++buttonId) {
        if (sceneId == l_scenes[buttonId][0]) {
            return TRUE;
        }
    }
    return FALSE;
}

/*..........................................................................*/
bool Scene_ScanScene(uint8_t sceneId, uint8_t idx)
{
    uint8_t buttonId;
    for (buttonId = 0; buttonId < BUTTON_LAST; ++buttonId) {
        if (sceneId == l_scenes[buttonId][idx]) {
            return TRUE;
        }
    }
    return FALSE;
}

/*..........................................................................*/
uint8_t Scene_GetButtonForScene(uint8_t scene)
{
    switch (scene) {
    case T1_S0:
    case T1_S1:
    case T1_DEC:
    case T1_INC:
    case T1_STOP_S:
        return BUTTON_GROUP_1;
    case T2_S0:
    case T2_S1:
    case T2_DEC:
    case T2_INC:
    case T2_STOP_S:
        return BUTTON_GROUP_2;
    case T3_S0:
    case T3_S1:
    case T3_DEC:
    case T3_INC:
    case T3_STOP_S:
        return BUTTON_GROUP_3;
    case T4_S0:
    case T4_S1:
    case T4_DEC:
    case T4_INC:
    case T4_STOP_S:
        return BUTTON_GROUP_4;
    }
    return 0;
}
